searchData={"items":[{"type":"module","title":"Application.Module","doc":"","ref":"Application.Module.html"},{"type":"macro","title":"Application.Module.defbehaviour/1","doc":"","ref":"Application.Module.html#defbehaviour/1"},{"type":"macro","title":"Application.Module.defimplementation/1","doc":"","ref":"Application.Module.html#defimplementation/1"},{"type":"type","title":"Application.Module.t/0","doc":"","ref":"Application.Module.html#t:t/0"},{"type":"module","title":"Application.Module.Macros","doc":"This module contains all the macros privately used by the `Application.Module` module.\nBecause we don't want these macros to be user-facing when users do `use Application.Module`,\nwe extract them into a different module.","ref":"Application.Module.Macros.html"},{"type":"macro","title":"Application.Module.Macros.define_behaviour_functions/1","doc":"This macro returns an AST that defines the functions that are defined in the behaviour module.","ref":"Application.Module.Macros.html#define_behaviour_functions/1"},{"type":"function","title":"Application.Module.Macros.ensure_module_loaded!/1","doc":"This macro ensures the given module has been loaded.","ref":"Application.Module.Macros.html#ensure_module_loaded!/1"},{"type":"macro","title":"Application.Module.Macros.functions/0","doc":"This macro returns an AST containing a set of utility functions\nthat need to be incorporated into the module that uses the `Application.Module` module.","ref":"Application.Module.Macros.html#functions/0"},{"type":"macro","title":"Functions - Application.Module.Macros.functions/0","doc":"- `mock_module/0`: Returns the mock module.\n- `implementation_module/0`: Returns the implementation module.\n- `behaviour_module/0`: Returns the behaviour module.\n- `get_application_env_module/0`: Returns the module that has been set in the environment (if any).","ref":"Application.Module.Macros.html#functions/0-functions"},{"type":"module","title":"Application.Module.NestedKeywordList","doc":"This module provides utilities to generate nested keyword lists.","ref":"Application.Module.NestedKeywordList.html"},{"type":"function","title":"Application.Module.NestedKeywordList.generate_nested_keyword_list/2","doc":"","ref":"Application.Module.NestedKeywordList.html#generate_nested_keyword_list/2"},{"type":"extras","title":"application_module","doc":"# `application_module`\n\nThe Application Module serves as a utility library designed to facilitate the creation of modules that conform to a specific behavior. Additionally, it allows for the behavior implementation to be managed through the application's configuration settings. This library is particularly useful when paired with testing libraries like [Mox](https://github.com/dashbitco/mox) or [Hammox](https://github.com/msz/hammox), as it helps eliminate the boilerplate code often required to retrieve the module ID from the application environment.\n\nThe following is an exampmle of an application module:\n\n```elixir\ndefmodule MyApplication.Module do\n  use Application.Module\n\n  defbehaviour do\n    @callback say_hello(name :: String.t()) :: String.t()\n  end\n\n  defimplementation do\n    def say_hello(name) do\n      \"Hello, #{name}!\"\n    end\n  end\nend\n```\n\n`Application.Module` implements each of the behaviour functions in `MyApplication.Module`. Unless overriden by the application's configuration `[:my_application, :modules, :module]`, the functions will delegate the execution to the implementations in the module `MyApplication.Module.Implementation`, which is automatically generated by `Application.Module` with the body of the `defimplementation` block.\n\nWhen used in companion with a mocking library like Mox, you can easily mock the module's implementation in your tests:\n\n```elixir\nMox.defmock(MyApplication.Module.Mock, for: MyApplication.Module.Behaviour)\nApplication.put_env(:my_application, :modules, [module: MyApplication.Module.Mock])\n```\n\n> **Note:** That the naming of the modules and the key of the module under the application's configuration is conventional.","ref":"readme.html"},{"type":"extras","title":"Installation - application_module","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `application_module` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:application_module, \"~> 0.3.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation"}],"content_type":"text/markdown"}